class Sitemap
  attr_reader :entries, :collection_scope

  def initialize(entries = [], **options, &block)
    @entries          = entries
    @collection_scope = options[:collection_scope] || :all
    yield self if block_given?
  end

  def entry(location, **options)
    @entries << Entry.new(location, **options)
  end

  def collection(resource, record_route, collection_route = nil)
    records = resource.send(collection_scope).inject [] do |m, record|
      m << Entry.new(record_route[record],
        lastmod: record.respond_to?(:updated_at) ? record.updated_at : nil
      )
    end
    if collection_route
      @entries << collection_entry(collection_route, records)
    end
    @entries += records
  end

  def each_entry(&block)
    @entries.each &block
  end

private

  def collection_entry(location, records, options = {})
    options.merge! lastmod: records.max_by(&:lastmod).lastmod if records.any?
    Entry.new(location, options)
  end

  class Entry
    attr_reader :location, :lastmod, :changefreq, :priority

    def initialize(location, lastmod: nil, changefreq: nil, priority: nil)
      @location   = location
      @lastmod    = lastmod
      @changefreq = changefreq
      @priority   = priority
    end
  end
end
